// 单向链表的插入与创建
// 要点：
//      通过调用单向链表的节点插入函数来创建单向链表
// 关键字：幽灵数组，单向链表，函数调用

#include <stdlib.h>
#define LIST_MAXLEN 1000  // 链表最大长度

typedef struct node { 
    struct node *next; 
    int data; 
}Node;
///////////////////////////////形状声明：结构体自引用next域构成单向链表///////////////////////////////
//@ shape next: list;

///////////////////////////////逻辑变量和幽灵变量（数组）声明////////////////////////////////////////
//@ logic Node* oldhead;
//@ ghost int position;
//@ ghost int olddata[];


////////////////////////////////////////listInsert函数的验证////////////////////////////////////////
/*@
requires
    head == \null && oldhead == head && position == 0 
  ||
    \list(head) && \length(head, next) == position 
    && oldhead == head && position > 0 && position <= LIST_MAXLEN 
    && (\forall integer i:[0..position-1].olddata[i] == head->(next:i)->data)
    ;
exits \exit_status == 1;
assigns olddata[0..position], *oldhead;
ensures
    \list(\result) && \length(\result, next) == position+1 
    && oldhead == \null && position == 0 && \result->data == data 
  ||
    \list(\result) && \length(\result, next) == position+1 
    && oldhead == \result && position > 0 && \result->(next:position)->data == data 
    && (\forall integer i:[0..position-1].olddata[i] == \result->(next:i)->data)
    ;
*/
Node* listInsert(Node* head, const int data){
    Node *ptr, *ptr1, *p;
    int j;
    p = (Node*)malloc(sizeof(Node));
    if (p == NULL) {
    exit(1);
    }
    p->data = data; 
    p->next = NULL;
    if (head == NULL) {
    head = p;
    //@ ghost olddata[0] = head->data;
    } else {
        ptr1 = head;
        ptr = head->next;
        j = 0;
        /*@ loop invariant 
                \list(head) && head == oldhead 
                && \length(head, next) == position 
                && \length(p, next) == 1 && \list(p) && p->next == \null 
                && 0 < position <= LIST_MAXLEN && ptr1 == head->(next:j) 
                && ptr == ptr1->next && 0 <= j <= position-1 && p->data == data 
                && (\forall integer i:[0..position-1].olddata[i] == head->(next:i)->data)
              ||
                oldhead == head && \list_seg(head, ptr1) 
                && ptr1 == head->(next:j) && ptr1->next == \null 
                && ptr == \null 
                && 0 < position <= LIST_MAXLEN && j == position-1 
                && \length(p, next) == 1 && \list(p) && p->next == \null 
                && p->data == data 
                && (\forall integer i:[0..position-1].olddata[i] == head->(next:i)->data)
                ;
        */
        while(ptr != NULL){
            ptr1 = ptr; 
            ptr = ptr->next;
            j = j+1;
        }
        p->next = ptr1->next;  
        ptr1->next = p;
        //@ ghost olddata[j+1] = data;
    }
    return head;
}

////////////////////////////////////////listCreate函数的验证////////////////////////////////////////
/*@
requires
    head == \null && oldhead == head 
    && length > 0 && length <= LIST_MAXLEN 
    && (\forall int i:[0..length-2].array[i] <= array[i+1]) 
    && \length(array) == length && \offset(array) == 0
    ;
exits \exit_status == 1;
assigns position,olddata[0..length-1];
ensures
    \list(\result) && \length(\result, next) == length 
    && oldhead == \null && length > 0 && length <= LIST_MAXLEN 
    && \length(array) == length && \offset(array) == 0 
    && (\forall int i:[0..length-1].\result->(next:i)->data == array[i])
    ;
*/
Node* listCreate(Node* head, const int * const array, const int length) {
    int i = 0;
    //@ ghost position = 0;
    /*@ loop invariant 
            //1.循环入口
            oldhead == \null && \null == head 
            && i == 0 && \length(array) == length 
            && \offset(array) == 0 && length > 0 
            && length <= LIST_MAXLEN && position == 0 
          ||
            //2.循环中及循环出口
            \list(head) && oldhead == \null && \length(head, next) == i 
            && \length(array) == length && 0 < i <= length <= LIST_MAXLEN 
            && \offset(array) == 0 && position == i 
            && head->(next:i-1)->data == array[i-1] 
            && (\forall int j:[0..i-1].head->(next:j)->data == array[j] == olddata[j])
            ;
    */
    while(i<length){
        head = listInsert(head,array[i]);
        //@ ghost olddata[i] = array[i];
        //@ ghost position = position+1;
        i = i+1;
    }
    return head;
}